﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace WindowsFormsApplication1 {
	public partial class mainForm : Form {

		// RNG
		private int seed = 55555;
		private Random randy;
		private Random textrandy;

		// bar values
		private int currentProgress = 0;
		private int currentIncrement = 1;
		private bool direction = true; // determines which direction the bar travels (true == normal)

		// stats
		private uint completedBars = 0;
		private uint clicksTotal = 0;
		private uint clicksUseful = 0;

		// timing
		private int timerMainValue = 0;
		private int timerIntervalChangeValue = 0;
		private DateTime startTime = DateTime.Now;
		private int lastTick;

		// misc
		private List<String[]> loadingText = new List<String[]>();
		private object ActiveProgressBar;
		private bool mouseinpanel = false;
		private Point lastmouse = Cursor.Position;

		public mainForm(String[] args) {

			if (args.Length >= 1) {
				try {
					seed = int.Parse(args[0]);
				} catch (Exception) {
					// oh well, keep default seed
				}
			}

			randy = new Random(seed);
			textrandy = new Random(seed);

			loadingText.Add(new String[] { "Loading", "Unloading" });
			loadingText.Add(new String[] { "Still Loading", "Still Unloading" });
			loadingText.Add(new String[] { "Pre-buffering", "Post-debuffering" });
			loadingText.Add(new String[] { "Buffering", "Debuffering" });
			loadingText.Add(new String[] { "Reticulating Splines", "Unreticulating Splines" });
			loadingText.Add(new String[] { "Re-Reticulating Splines", "Un-Unreticulating Splines" });
			loadingText.Add(new String[] { "Eating Children", "Regurgitating Children" });
			loadingText.Add(new String[] { "Increasing Progress Bar", "Reducing Progress Bar" });
			loadingText.Add(new String[] { "BRB", "" });
			loadingText.Add(new String[] { "Caching Items", "Uncaching Items" });
			loadingText.Add(new String[] { "Moving Files", "Undoing Moved Files" });
			loadingText.Add(new String[] { "Copying Files", "Deleting Copied Files" });
			loadingText.Add(new String[] { "Deleting Temporary Files", "Restoring Temporary Files" });
			loadingText.Add(new String[] { "Unzipping", "Zipping" });
			loadingText.Add(new String[] { "Extracting", "Unextracting" });
			loadingText.Add(new String[] { "Extracting xpack2.rfa", "Compacting xpack2.rfa" });
			loadingText.Add(new String[] { "Decompressing", "Compressing" });
			loadingText.Add(new String[] { "Compressing", "Decompressing" });
			loadingText.Add(new String[] { "Waiting", "Not Waiting" });
			loadingText.Add(new String[] { "Still Waiting", "Still Not Waiting" });
			loadingText.Add(new String[] { "Encrypting", "Unencrypting" });
			loadingText.Add(new String[] { "Decrypting", "Undecrypting" });
			loadingText.Add(new String[] { "Calculating", "" });
			loadingText.Add(new String[] { "Premeditating", "Postmeditating" });
			loadingText.Add(new String[] { "Preparing", "Unpreparing" });
			loadingText.Add(new String[] { "Preparing For Extraction", "Unpreparing For Extraction" });
			loadingText.Add(new String[] { "Preparing To Prepare", "Unpreparing To Unpreparing" });
			loadingText.Add(new String[] { "Thank You For Waiting", "Thank You For Waiting" });
			loadingText.Add(new String[] { "Please Wait", "Please Wait" });
			loadingText.Add(new String[] { "Connecting", "Disconnecting" });
			loadingText.Add(new String[] { "Testing", "" });
			loadingText.Add(new String[] { "Testing Connection", "Testing Disconnection" });
			loadingText.Add(new String[] { "Assessing", "Unassessing" });
			loadingText.Add(new String[] { "Creating More Progress Bars", "Deleting Progress Bars" });
			loadingText.Add(new String[] { "Making Progress", "Deleting Progress" });
			loadingText.Add(new String[] { "Gathering Data", "Ungathering Data" });
			loadingText.Add(new String[] { "Sending Data", "Unsending Data" });
			loadingText.Add(new String[] { "Applying Data", "Unapplying Data" });
			loadingText.Add(new String[] { "Applying Settings", "Unapplying Settings" });
			loadingText.Add(new String[] { "Writing To Registry", "Deleting From Registry" });
			loadingText.Add(new String[] { "Removing Temporary Registry Entries", "Restoring Temporary Registry Entries" });

			InitializeComponent();
			timerRunTime.Tick += new EventHandler(timerRunTime_Tick);

			ActiveProgressBar = progressMain;

			lastTick = DateTime.Now.Millisecond;

			HookEvents();
		}

		private void HookEvents() {
			foreach (Control ctl in this.Controls) {
				ctl.MouseClick += new MouseEventHandler(mainForm_MouseClick);
				ctl.MouseDoubleClick += new MouseEventHandler(mainForm_MouseClick);
				ctl.MouseMove += new MouseEventHandler(mainForm_MouseMove);
			}

			MouseClick += new MouseEventHandler(mainForm_MouseClick);
			MouseDoubleClick += new MouseEventHandler(mainForm_MouseClick);
			//MouseMove += new MouseEventHandler(mainForm_MouseMove);
		}

		void mainForm_MouseMove(object sender, MouseEventArgs e) {

			Point panelLoc = panelBarPosition.Location;
			panelLoc.Offset(this.Location);
			panelLoc.Offset(new Point(8, 30));

			Rectangle boundingBox = new Rectangle(panelLoc, panelBarPosition.Size);

			if (boundingBox.Contains(Cursor.Position) && !mouseinpanel) { // mouse entered area


				// if mouse has entered from left or right side
				if ((lastmouse.X < boundingBox.X && Cursor.Position.X >= boundingBox.X) ||
					(lastmouse.X > boundingBox.Right && Cursor.Position.X <= boundingBox.Right)) {

					timerMain.Interval = Math.Max(timerMain.Interval - 2, 1);
				}

				mouseinpanel = true;

			} else if (!boundingBox.Contains(Cursor.Position) && mouseinpanel) { // mouse left area

				mouseinpanel = false;
			}

			lastmouse = Cursor.Position;
			updateStatusText();
		}

		void mainForm_MouseClick(object sender, MouseEventArgs e) {

			if (sender == ActiveProgressBar) {
				timerMain.Interval = Math.Max(timerMain.Interval - 1, 25);
				clicksUseful++;
			} else if (sender == buttonCancel) {
				timerMain.Interval += 10;
				timerIntervalChange.Interval += 100;
			}

			Point panelLoc = panelBarPosition.Location;
			panelLoc.Offset(this.Location);
			panelLoc.Offset(new Point(8, 30));

			Rectangle boundingBox = new Rectangle(panelLoc, panelBarPosition.Size);

			if (boundingBox.Contains(Cursor.Position)) {

				timerMain.Enabled = false;
				timerMain.Interval = Math.Max(timerMain.Interval - 5, 25);
				timerMain.Enabled = true;

				if (sender != ActiveProgressBar)
					clicksUseful++;
			}

			clicksTotal++;
			updateStatusText();
		}

		void timerRunTime_Tick(object sender, EventArgs e) {

			TimeSpan currentTime = DateTime.Now.Subtract(startTime);

			// this is a workaround for the behavior of timers
			// since setting the interval of a timer resets it's internal counter
			// i just keep my own counter and only reset if the time has elapsed
			int tickDelta = Math.Max(DateTime.Now.Millisecond - lastTick, 0);
			timerMainValue += tickDelta;
			timerIntervalChangeValue += tickDelta;
			lastTick = DateTime.Now.Millisecond;

			if (timerMainValue >= timerMain.Interval) {
				timerMain_Tick(sender, e);
				timerMainValue = (int)((float)timerMainValue % (float)timerMain.Interval);
			}

			if (timerIntervalChangeValue >= timerIntervalChange.Interval) {
				timerIntervalChange_Tick(sender, e);
				timerIntervalChangeValue = (int)((float)timerIntervalChangeValue % (float)timerIntervalChange.Interval);
			}

			labelTime.Text = String.Format("Time: {0:D2}:{1:D2}:{2:D2}:{3:D2}:{4:D3}",
				currentTime.Days, currentTime.Hours,
				currentTime.Minutes, currentTime.Seconds, currentTime.Milliseconds);
		}

		void timerIntervalChange_Tick(object sender, EventArgs e) {

			int maxTime = 500;
			int chance = randy.Next(100);

			if (chance <= 50) { // 50% chance for 100 or less
				maxTime = 100;
			} else if (chance > 50 && chance <= 75) { // 25% chance for 500 or less
				maxTime = 500;
			} else { // 25% chance for 1000 or less
				maxTime = 1000;
			}

			timerMain.Interval = (50 + randy.Next(maxTime - 50));

			timerIntervalChange.Interval = (2000 + randy.Next(10000));


			updateStatusText();

			int textset = (direction ? 0 : 1);

			String randText = loadingText[textrandy.Next(loadingText.Count)][textset];

			while (randText == labelProgressText.Text || randText == "")
				randText = loadingText[textrandy.Next(loadingText.Count)][textset];

			labelProgressText.Text = randText + "...";
		}

		void timerMain_Tick(object sender, EventArgs e) {

			if (!direction)
				currentIncrement = currentIncrement * -1;

			if (currentProgress <= 1 && completedBars == 0 && !direction)
				this.Close();

			//if (currentProgress == 1 && completedBars > 0 && !direction)
				//currentProgress = 100;

			// if progress is at 95 to 99%, with 25% chance, pause for 2 to 5 seconds
			if (currentProgress >= 95 && currentProgress <= 99 && randy.Next(3) == 0) {

				timerMain.Interval = 2000 + randy.Next(3000);
				timerMainValue = 0;
				timerIntervalChange.Interval += timerMain.Interval;
			}

			if (direction) {

				if (currentProgress < 100 && (currentProgress + currentIncrement) >= 100) {
					timerMain.Interval = 3000;
					timerMainValue = 0;
					timerIntervalChange.Interval = 3000;
					timerIntervalChangeValue = 0;
					completedBars++;
					labelCompleted.Text = "Completed: " + completedBars;

					currentProgress = 100;
				} else {
					currentProgress = (int)((currentProgress + currentIncrement) % 100);
				}
			} else {

				if (currentProgress == 1 && completedBars > 0) {
					completedBars--;
					currentProgress = 100;
				}

				if (currentProgress > 1 && (currentProgress + currentIncrement) <= 1) {
					timerMain.Interval = 1000;
					timerMainValue = 0;
					timerIntervalChange.Interval = 1000;
					timerIntervalChangeValue = 0;

					currentProgress = 1;
				} else {
					currentProgress = (currentProgress + currentIncrement);
				}
			}


			if (currentProgress < 100) {
				progressMain.Value = currentProgress + 1;
				progressMain.Value = currentProgress;
			} else {
				progressMain.Value = 100;
				progressMain.Value = 99;
				progressMain.Value = 100;
			}

			int start = progressMain.Location.X + this.Location.X;
			int end = start + progressMain.Size.Width;

			float nextX = (((float)progressMain.Value / 100) * (float)(end - start)) - (panelBarPosition.Width / 4);

			panelBarPosition.Location = new Point((int)nextX, progressMain.Location.Y - 3);

			currentIncrement = 1;
			updateStatusText();
		}

		private void updateStatusText() {
			labelMain.Text = progressMain.Value.ToString() + "%";
			labelDebug.Text = "" +
				//"Main: " + timerMain.Interval +
				//" | Change: " + timerIntervalChange.Interval +
				//" | Increment: " + currentIncrement +
				"Clicks: " + clicksTotal +
				" | Useful Clicks: " + clicksUseful +
				" | Seed: " + seed;

			this.Text = labelProgressText.Text + " " + labelMain.Text;

			labelCompleted.Text = "Completed: " + completedBars;
		}

		private void exitToolStripMenuItem_Click(object sender, EventArgs e) {

			DialogResult result = MessageBox.Show("Are you sure you want to exit?\nYou will lose all of your progress.", "Exit", MessageBoxButtons.OK, MessageBoxIcon.Question);

			if (result == DialogResult.OK) {
				direction = !direction;
				timerIntervalChange_Tick(sender, e);
			}
		}
	}
}
